<?php

/**
 * @file
 * Admin pages for the Zeitgeist module.
 *
 * @copyright (c) 2005-2013 Ouest Systemes Informatiques (OSInet)
 *
 * @license Licensed under the General Public License version 2 or later.
 */

use Drupal\zeitgeist\Block;
use Drupal\zeitgeist\BlockLatest;
use Drupal\zeitgeist\BlockTop;
use Drupal\zeitgeist\Cache;
use Drupal\zeitgeist\DisplayType;
use Drupal\zeitgeist\ListBlock;
use Drupal\zeitgeist\PageType;
use Drupal\zeitgeist\Sanitization;
use Drupal\zeitgeist\Statistics;

/**
 * Implement a settings form.
 *
 * @return array
 *   An array containing form items to place on the module settings page.
 */
function _zeitgeist_admin_settings($form, $form_state) {
  drupal_add_js(drupal_get_path('module', 'zeitgeist') . '/zeitgeist.js');

  $form[Statistics::VARHISTORY] = array(
    '#type'          => 'textfield',
    '#title'         => t('Number of days of history'),
    '#description'   => t('Historical data are kept for that number of days. 0 means forever. Pruning is performed by cron on a daily basis'),
    '#default_value' => Statistics::getHistoryDays(),
    // This is good until which 2037, should be enough for all 7.x needs.
    '#size'          => 4,
    '#maxlength'     => 4,
  );

  $form['zeitgeist-display'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Display settings'),
    '#collapsible'   => TRUE,
  );

  // Block configure sub-forms.
  foreach (array_keys(zeitgeist_block_info()) as $delta) {
    $form['zeitgeist-display']["zeitgeist-$delta"] = Block::instance($delta)->configure();
  }

  // Report configure sub-form.
  $page_settings = _zeitgeist_admin_settings_page($form_state);
  $form['zeitgeist-display']['zeitgeist-page'] = $page_settings['form'];
  $menu_rebuild = $page_settings['menu_rebuild'];

  // Search filtering fieldset.
  $form['zeigeist-filtering'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Search filtering'),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE,
  );
  $form['zeigeist-filtering'][Sanitization::VARVALIDATION] = array(
    '#type'          => 'radios',
    '#title'         => t('How should search strings be validated to prevent spamming ZG blocks ?'),
    '#default_value' => Sanitization::getValidationType(),
    '#options'       => array(
      Sanitization::NO      => t('No validation performed: record and display every search'),
      Sanitization::RECORD  => t('Do not record searches containing HTML'),
      Sanitization::DISPLAY => t('Do not display searches containing HTML, but store them'),
    ),
    '#description'   => t('<p>Recommended value is "Do not record HTML": spammers may abuse ZG by submitting links via scripts.</p>'),
  );
  $form['zeigeist-filtering'][ZGVARRECORDEMPTY] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Should Zeitgeist record empty searches ?'),
    '#default_value' => variable_get(ZGVARRECORDEMPTY, ZGDEFRECORDEMPTY),
    '#description'   => t('Zeitgeist normally records empty searches, but some of them can be spurious and not represent actual searches, but just clicks on search links by spiders, and some webmasters can wish to ignore them. Whatever this setting, they are never displayed in the "latest searches" block.'),
  );

  $form['advanced'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Advanced settings'),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE,
  );

  $arguments = array('!attr' => '<code>rel="nofollow"</code>');
  $form['advanced'][ListBlock::VARNOFOLLOW] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Apply !attr to links on the "recent searches" and "top searches" blocks', $arguments),
    '#default_value' => ListBlock::isNoFollowEnabled(),
    '#return_value'  => 1,
    '#description'   => t('An attribute for the "a" (X)HTML alement, !attr is non-standard, but is recognized by several search engines, notably Google, MSN and Yahoo!', $arguments),
  );

  $modules = system_rebuild_module_data();
  $name = 'zeitgeist';
  $info = $modules[$name];
  $form['advanced']['version'] = array(
    '#markup' => '<p>' . t('This site is running %module version [%version], schema version: [%schema].', array(
      '%module' => $name,
      '%version' => isset($info->info['version']) ? $info->info['version'] : t('Undefined'),
      '%schema' => $info->schema_version,
    )) . '</p>',
  );

  // Handlers cannot go 'Foo::bar' or array('Foo', 'bar'), but can be closures.
  // @see form_execute_handlers()
  $form['#submit'][] = function () use ($menu_rebuild) {
    Cache::clearAll();

    // We changed the value of $(PageType::getVar()), changing a menu entry.
    if ($menu_rebuild) {
      variable_set('menu_rebuild_needed', TRUE);
    }
  };

  $form = system_settings_form($form);
  return $form;
}

/**
 * Form helper: settings for the Zeitgeist report pages.
 *
 * @param $form_state
 *   The form state array.
 *
 * @return array
 *   A hash containing:
 *   - form: the sub-form array for the module settings form.
 *   - menu_rebuild: a flag mentioning whether the menu should be rebuilt.
 */
function _zeitgeist_admin_settings_page($form_state) {
  $form = array(
    '#type'          => 'fieldset',
    '#title'         => t('Default zeitgeist page'),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE,
  );
  $form[ZGVARPAGEHEIGHT]  = array(
    '#type'          => 'textfield',
    '#title'         => t('Height of the zeitgeist pager'),
    '#default_value' => variable_get(ZGVARPAGEHEIGHT, ZGDEFPAGEHEIGHT),
    '#size'          => 4,
    '#maxlength'     => 4,
    '#element_validate' => array('element_validate_integer_positive'),
  );
  $form[ZGVARPAGEDAYS] = array(
    '#type'          => 'textfield',
    '#title'         => t('Days on the <a href="!link">Zeitgeist report</a> page', array(
      '!link' => check_url(url(ZGPATHREPORT)),
    )),
    '#size'          => 4,
    '#maxlength'     => 4,
    '#default_value' => (int) variable_get(ZGVARPAGEDAYS, ZGDEFPAGEDAYS),
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $page_types = PageType::getConstants();
  $current_page_type = PageType::getActiveReport();
  $submitted_page_type = isset($form_state['input'][PageType::getVar()])
    ? $form_state['input'][PageType::getVar()]
    : $current_page_type;

  $form[PageType::getVar()] = array(
    '#type'          => 'radios',
    '#title'         => t('Report page type'),
    '#description'   => t('Note: the "views" default report ignores Zeitgeist settings and uses the View configuration instead.'),
    '#default_value' => $current_page_type,
    '#options'       => array_combine($page_types, $page_types),
  );

  $menu_rebuild = ($current_page_type != $submitted_page_type);
  $ret = compact('form', 'menu_rebuild');
  return $ret;
}
