<?php
/**
 * @file
 * Report pages for the Zeitgeist module.
 *
 * While usable, these three pages should mostly be considered as examples.
 * For smarter results, build your own page with Statistics::getStatistics()
 * and the Span::* human-oriented duration constants, or use the Views 3
 * integration as show in zeitgeist_page_results_views(). Either way, this will
 * free you from having to build the database query yourself.
 *
 * @see Drupal\zeitgeist\Statistics::getStatistics()
 *
 * @copyright (c) 2005-2013 Ouest Systemes Informatiques (OSInet)
 *
 * @license Licensed under the General Public License version 2 or later.
 */

use Drupal\zeitgeist\Statistics;

/**
 * Page callback showing ZG results.
 *
 * @param int $days
 *   The number of days over which to fetch Zeitgeist.
 *
 * @return string
 *   Rendered page content.
 */
function zeitgeist_page_results_multicolumn($days = 0) {
  if (!is_numeric($days) || $days <= 0) {
    $days = variable_get(ZGVARPAGEDAYS, ZGDEFPAGEDAYS);
  }
  $limit = REQUEST_TIME - Statistics::ONEDAY * $days;
  drupal_set_title(t('Zeitgeist over the last @days days', array('@days' => $days)), PASS_THROUGH);

  // Pager height.
  $height = (int) variable_get(ZGVARPAGEHEIGHT, ZGDEFPAGEHEIGHT);

  $ar_searches = array();
  $sql = <<<SQL
SELECT DISTINCT zg.search, count(zg.ts) AS nts
FROM {zeitgeist} zg
WHERE zg.ts >= :ts
GROUP BY zg.search
ORDER BY nts DESC, zg.search ASC
SQL;

  // No access control on this table.
  $results = db_query_range($sql, 0, $height * 3, array(':ts' => $limit));
  foreach ($results as $result) {
    if ($result->search == '') {
      $result->search = '&lt;vide&gt;';
    }
    $ar_searches[$result->search] = $result->nts;
  }

  $results_count = count($ar_searches);
  $break = $results_count / 3;
  $rows = array();
  $offset_in_column = 0;
  foreach ($ar_searches as $search => $count) {
    if ($offset_in_column < $break) {
      $rows[$offset_in_column] = array(
        $search, $count,
        NULL, NULL,
        NULL, NULL,
      );
    }
    elseif ($offset_in_column < 2 * $break) {
      $rows[$offset_in_column - $break][2] = $search;
      $rows[$offset_in_column - $break][3] = $count;
    }
    else {
      $rows[$offset_in_column - 2 * $break][4] = $search;
      $rows[$offset_in_column - 2 * $break][5] = $count;
    }
    $offset_in_column++;
  }

  $header = array(
    t('Search'), t('#'),
    t('Search'), t('#'),
    t('Search'), t('#'),
  );
  $attributes = array();

  $ret = array();
  $ret['items'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => $attributes,
  );
  $ret['note'] = array(
    '#markup' => '<p>' . t('Searches below the pager <a href="!limit">limit</a> are not included in this list.',
      array('!limit' => url(ZGPATHSETTINGS))) . "</p>\n",
  );
  return $ret;
}

/**
 * Sample basic stats page.
 *
 * @return string
 *   HTML
 */
function zeitgeist_page_results_simple($days = 0) {
  if (!is_numeric($days) || $days <= 0) {
    $days = (int) variable_get(ZGVARPAGEDAYS, ZGDEFPAGEDAYS);
  }
  $limit = REQUEST_TIME - Statistics::ONEDAY * $days;
  drupal_set_title(t('Zeitgeist over the last @days days', array('@days' => $days)), PASS_THROUGH);

  // Pager height.
  $height = variable_get(ZGVARPAGEHEIGHT, ZGDEFPAGEHEIGHT);

  // Do not use positional groupBy(), as this breaks PagerDefault.
  $query = db_select(Statistics::TABLE, 'zg')
    ->extend('PagerDefault')
    ->fields('zg', array('search'));
  $query->addExpression('COUNT(zg.ts)', 'cnt');
  $query->condition('zg.category', 'node')
    ->condition('zg.ts', $limit, '>')
    ->limit($height)
    ->groupBy('search')
    ->orderBy(2, 'DESC')
    ->orderBy(1, 'ASC');
  $results = $query->execute();

  $items = array();
  $cnt = 0;
  foreach ($results as $result) {
    $items[] = "$result->cnt - " . l($result->search, "search/node/$result->search");
    $cnt += $result->cnt;
  }

  $ret = array();
  $ret['items'] = array(
    '#theme' => 'item_list',
    '#items' => $items,
  );
  $ret['summary'] = array(
    '#markup' => '<p>' . t('For a total of %cnt hits for these %height searches.', array(
      '%cnt' => $cnt,
      '%height' => $height,
    )) . "</p>\n",
  );

  // Attach the pager theme.
  $ret['pager_pager'] = array('#theme' => 'pager');
  return $ret;
}

/**
 * Page callback: sample Views-based report.
 *
 * @see zeitgeist_preprocess_views_view()
 *
 * @return string
 *   HTML
 */
function zeitgeist_page_results_views() {
  $ret = views_embed_view('zeitgeist');
  return $ret;
}
